﻿using System;
using SDX.Compiler;
using Mono.Cecil;
using Mono.Cecil.Cil;
using System.Linq;
using System.Collections.Generic;

public class GameStageFix : IPatcherMod
{
    public bool Link(ModuleDefinition gameModule, ModuleDefinition modModule)
    {
        InsertGameStageDefinitionRead(gameModule, modModule);
        InsertGetGameStage(gameModule, modModule);
        return true;
    }
    public void InsertGetGameStage(ModuleDefinition gameModule, ModuleDefinition modModule)
    {
        TypeDefinition entityPlayer = gameModule.Types.First(d => d.Name == "EntityPlayer");
        MethodDefinition entityPlayerGetGameStage = entityPlayer.Methods.First(d => d.Name == "get_gameStage");

        TypeDefinition gameStageDef = gameModule.Types.First(d => d.Name == "GameStageDefinition");
        FieldDefinition gameStageModeField = gameStageDef.Fields.First(d => d.Name == "GameStageMode");

        ILProcessor proc1 = entityPlayerGetGameStage.Body.GetILProcessor();
        while (proc1.Body.Variables.Count > 0)
        {
            proc1.Body.Variables.RemoveAt(0);
        }
        while (proc1.Body.Instructions.Count > 0)
        {
            proc1.Body.Instructions.RemoveAt(0);
        }
        proc1.Body.Instructions.Add(Instruction.Create(OpCodes.Ldarg_0));
        proc1.Body.Instructions.Add(Instruction.Create(OpCodes.Ldsfld, gameStageModeField));

        TypeDefinition gameStageFixMethod = modModule.Types.First(d => d.Name == "GameStageFixMethod");
        MethodDefinition getGameStage = gameStageFixMethod.Methods.First(d => d.Name == "GetGameStage");
        MethodReference getGameStageRef = gameModule.Import(getGameStage);

        proc1.Body.Instructions.Add(Instruction.Create(OpCodes.Call, getGameStageRef));
        proc1.Body.Instructions.Add(Instruction.Create(OpCodes.Ret));


    }
    public void InsertGameStageDefinitionRead(ModuleDefinition gameModule, ModuleDefinition modModule)
    {
        TypeDefinition classDef = gameModule.Types.First(d => d.Name == "GameStagesFromXml");
        MethodDefinition methodDef = classDef.Methods.First(d => d.Name == "Load");

        TypeDefinition gameStageDef = gameModule.Types.First(d => d.Name == "GameStageDefinition");
        FieldDefinition gameStageModeField = gameStageDef.Fields.First(d => d.Name == "GameStageMode");
        Instruction insertPosition = null;
        foreach (Instruction inst in methodDef.Body.Instructions)
        {

            if (inst.OpCode == OpCodes.Stsfld && inst.Operand.GetType() == typeof(FieldDefinition) && ((FieldDefinition)inst.Operand).FullName == "System.Single GameStageDefinition::DaysAliveDifficultyBonus")
            {
                insertPosition = inst.Next;

            }
        }
        List<Instruction> jumpList = new List<Instruction>();
        foreach (Instruction inst in methodDef.Body.Instructions)
        {
            if (inst.OpCode == OpCodes.Brfalse_S && inst.Operand.GetType() == typeof(Instruction) && inst.Operand == insertPosition)
            {
                jumpList.Add(inst);
            }
        }

        ILProcessor proc1 = methodDef.Body.GetILProcessor();
        Instruction newJumpTo = Instruction.Create(OpCodes.Ldloc_S, proc1.Body.Variables[4]);
        proc1.InsertBefore(insertPosition, newJumpTo);//V_4 is XmlElement
        proc1.InsertBefore(insertPosition, Instruction.Create(OpCodes.Ldsflda, gameStageModeField));//ref GameOverMode 

        TypeDefinition gameStageFixMethod = modModule.Types.First(d => d.Name == "GameStageFixMethod");
        MethodDefinition setAddAttribute = gameStageFixMethod.Methods.First(d => d.Name == "SetAddAttribute");

        if (setAddAttribute == null)
        {
            Console.WriteLine("InsertCallBuffsFromXmlAdd setAddAttribute == null");
        }
        MethodReference setAddAttributeRef = gameModule.Import(setAddAttribute);
        Console.WriteLine("InsertCallBuffsFromXmlAdd setAddAttribute find");

        proc1.InsertBefore(insertPosition, Instruction.Create(OpCodes.Call, setAddAttributeRef));

        foreach (Instruction jump in jumpList)
        {
            jump.Operand = newJumpTo;
        }
    }

    public bool Patch(ModuleDefinition module)
    {
        Console.WriteLine("==MinWellnessZero Patcher===");
        AddGameStageDefinitionField(module);
        return true;
    }
    public void AddGameStageDefinitionField(ModuleDefinition module)
    {
        TypeDefinition multiBuffClass = module.Types.First(d => d.Name == "GameStageDefinition");
        TypeReference tr = module.Import(typeof(byte));
        AddFieldDefinition(multiBuffClass, "GameStageMode", FieldAttributes.Public| FieldAttributes.Static, tr);

    }

    public void AddFieldDefinition(TypeDefinition classDef, string fieldName, FieldAttributes fieldAttribute, TypeReference fieldType)
    {
        FieldDefinition fd = new FieldDefinition(fieldName, fieldAttribute, fieldType);
        classDef.Fields.Add(fd);

    }
    public void AddFieldDefinition(TypeDefinition classDef, string fieldName, FieldAttributes fieldAttribute, TypeReference fieldType, object constantvalue)
    {
        FieldDefinition fd = new FieldDefinition(fieldName, fieldAttribute, fieldType);
        fd.Constant = constantvalue;
        classDef.Fields.Add(fd);

    }

}
